'  ASM WAV Player Ver. 1b  (Modified for interactive play)
'  Plays only stereo, 16-bit PCM WAV files from SD card.
'  Copyright 2007 Raymond Allen  See end of file for terms of use.
'  Modified for Propeller BOE, and to accept serial input to control
'    the WAV clip to play.  

OBJ
    SD      :   "FSRW"
    debug   :   "FullDuplexSerial"
    SerIn   :   "FullDuplexSerial"  

CON _clkmode = xtal1 + pll16x
    _xinfreq = 5_000_000

    SD_basepin = 22         ' Base (first) pin for SD card
    RPin = 26               ' Right channel pin 
    LPin = 27               ' Left channel pin

    ' Set pins and baud rate for serial in
    SerIn_Rx = 0           ' Arduino to Propeller (don't forget 1K resistor!)
    SerIn_Tx = 1           ' Not used but defined anyway
    SerIn_Baud = 9600    

    ' Set pins and baud rate for Serial Terminal window
    debug_Rx = 31
    debug_Tx = 30
    debug_Baud = 115200

    buffSize = 100      

VAR long parameter1        ' @buff1 to ASM
    long parameter2        ' @buff2 to ASM
    long parameter3        ' sample rate to ASM
    long parameter4        ' #samples to ASM
    long buff1[buffSize]
    long buff2[buffSize]
    byte Header[44]
    long stack[50]         'Stack space for SerialIn
    byte cancelWAV 
    
PUB Main | n,i,j, SampleRate,Samples, cmd, WAVCOG,SERCOG 

  ' Initialize comm ports for serial input, monitor window
  SerIn.start(SerIn_Rx, SerIn_Tx, 0, SerIn_Baud)
  debug.start(debug_Rx, debug_Tx, 0, debug_Baud)

  ' Repeat forever
  repeat

    ' Empty buffer for data from serial input 
    SerIn.rxFlush
     
    repeat until (SerIn.rx == "!")         ' Wait for qualifier character
     
    if(SerIn.rx == "W")                    ' "W" command to play WAV

      debug.str(string("Starting Up", 13)) 
     
      ' Open the WAV file (only 16-bit PCM WAV Files) on SD card
      i:=sd.mount(SD_basepin)
      if (i<>0)
        repeat
       
      debug.str(string("SD Card Mounted", 13))
       
      ' Open file to use
      cmd := SerIn.rx
      debug.str(string("File number selected: "))
      debug.tx(cmd)
      debug.tx(13)
      case cmd
        "0":
          i:=sd.popen(string("test0.wav"), "r")  ' -- Play file for file #0 --
        "1":
          i:=sd.popen(string("test1.wav"), "r")  ' -- Play file for file #1 --
     
      if (i<>0)
        repeat
       
      ' Read data words to input stereo buffer
      i:=sd.pread(@Header, 44) 
      ' Get sample rate from header
      SampleRate:=Header[27]<<24+Header[26]<<16+Header[25]<<8+Header[24]
      debug.dec(SampleRate)
      debug.tx(13)
       
      ' Get # samples from header
      Samples:=Header[43]<<24+Header[42]<<16+Header[41]<<8+Header[40]
      Samples:=Samples>>2
      debug.dec(Samples)
      debug.tx(13)
       
      'Start ASM player in a new cog
      debug.str(string("Running ASM Player", 13))    
      parameter1:=@buff1[0]
      parameter2:=@buff2[0]
      parameter3:=clkfreq/SampleRate
      parameter4:=Samples
      WAVCOG := cognew(@ASMWAV,@parameter1)

      ' Start Serial In cog
      cancelWAV := 0
      SERCOG := cognew(SerialIn,@stack)

      ' Keep filling buffers until end of file
      '  Uses alternating buffers to always keep data ready
      n:=buffSize-1
      j:=buffsize*4                       'number of bytes to read
      repeat while (j==buffsize*4)        'repeat until end of file
        if (buff1[n]==0)
          j:=sd.pread(@buff1, buffSize*4) 'read data words to input stereo buffer   
       
        if (buff2[n]==0)
          j:=sd.pread(@buff2, buffSize*4) 'read data words to input stereo buffer  

        if (cancelWAV == 1)               'exit loop if cancel interrupted
          j := 0
          quit

      debug.str(string("Loop end", 13))        
      ' At end of file close SD, stop cogs
      SerIn.rxFlush      
      cogstop (SERCOG)      
      sd.pclose
      cogstop (WAVCOG)
      debug.str(string("Closing file", 13))

Pub SerialIn | cmd

  SerIn.rxFlush
  repeat
    repeat until (SerIn.rx == "!")       ' Wait for qualifier character
    cmd := SerIn.rx
    if (cmd == "X")                      ' Cancel
      SerIn.rx
      cancelWAV := 1
      quit

DAT
  ORG 0
ASMWAV
'load input parameters from hub to cog given address in par
        movd    :par,#pData1             
        mov     x,par
        mov     y,#4  'input 4 parameters
:par    rdlong  0,x
        add     :par,dlsb
        add     x,#4
        djnz    y,#:par

setup
        'setup output pins
        MOV DMaskR,#1
        ROL DMaskR,OPinR
        OR DIRA, DMaskR
        MOV DMaskL,#1
        ROL DMaskL,OPinL
        OR DIRA, DMaskL
        'setup counters
        OR CountModeR,OPinR
        MOV CTRA,CountModeR
        OR CountModeL,OPinL
        MOV CTRB,CountModeL
        'Wait for SPIN to fill table
        MOV WaitCount, CNT
        ADD WaitCount,BigWait
        WAITCNT WaitCount,#0
        'setup loop table
        MOV LoopCount,SizeBuff  
        'ROR LoopCount,#1    'for stereo
        MOV pData,pData1
        MOV nTable,#1
        'setup loop counter
        MOV WaitCount, CNT
        ADD WaitCount,dRate


MainLoop
        SUB nSamples,#1
        CMP nSamples,#0 wz
        IF_Z JMP #Done
        waitcnt WaitCount,dRate

        RDLONG Right,pData
        ADD Right,twos      'Going to cheat a bit with the LSBs here    
        MOV FRQA,Right
        ROL Right,#16       '16 LSBs are left channel
        MOV FRQB,Right
        WRLONG Zero,pData
        ADD pData,#4

        'loop
        DJNZ LoopCount,#MainLoop
        
        MOV LoopCount,SizeBuff        
        'switch table       ?
        CMP nTable,#1 wz
        IF_Z JMP #SwitchToTable2
SwitchToTable1
        MOV nTable,#1
        MOV pData,pData1
        JMP #MainLoop
SwitchToTable2
        MOV nTable,#2
        MOV pData,pData2
        JMP #MainLoop
        
                
Done
         'now stop
        COGID thisCog
        COGSTOP thisCog          

'Working variables
thisCog long 0
x       long 0
y       long 0
dlsb    long    1 << 9
BigWait long 100000
twos    long $8000_8000
        
'Loop parameters
nTable  long 0
WaitCount long 0
pData   long 0
LoopCount long 0
SizeBuff long buffsize
'Left    long 0
Right   long 0
Zero    long 0          

'setup parameters
DMaskR  long 0          'Right output mask
OPinR   long RPin       'Right channel output pin #    
DMaskL  long 0          'Left output mask 
OPinL   long LPin       'Left channel output pin #    
CountModeR long %00011000_00000000_00000000_00000000
CountModeL long %00011000_00000000_00000000_00000000


'input parameters
pData1   long 0         'Address of first data table        
pData2   long 0         'Address of second data table
dRate    long 5000      'clocks between samples
nSamples long 2000


{{
                            TERMS OF USE: MIT License

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
}}